<?php
/*
* Opinus.pl
*
*  @author Opinus.pl <kontakt@opinus.pl>
*  @copyright  2019  Opinus.pl
*/


if(!defined('_PS_VERSION_'))
  exit;

class Opinus extends Module {
	private $_html='';
	
	public function __construct(){
		$this->name = 'opinus';
		$this->tab = 'advertising_marketing';
		$this->version = '1.2';
		$this->author = 'Opinus.pl';
		$this->need_instance = 0;
		$this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_); 
		$this->bootstrap = true;
		
		parent::__construct();
		
		$this->displayName = $this->l('Opinus');
		$this->description = $this->l('Collect and manage customers\' reviews.');
		
		$this->confirmUninstall = $this->l('Are you sure you want to uninstall? Remember - after uninstalling mode your customers will not receive any more invitations for writing reviews. Your data on Opinus.pl will not be removed after uninstalling mode.');
	}
	
	private function executeSQL($requests){
		foreach($requests as $q){
			if(!DB::getInstance()->execute($q)){
				return false;
			}
		}
	}
	
	public function psversion(){
		$version=_PS_VERSION_;
		$exp=$explode=explode(".",$version);
		return $exp[1];
	}
	
	public function install(){
		if(Shop::isFeatureActive()){
			Shop::setContext(Shop::CONTEXT_ALL);
		}
		
		//adding extra column in databe table for module
		$sql=array();
		$sql[]="ALTER TABLE "._DB_PREFIX_."orders ADD opinus BOOLEAN NOT NULL DEFAULT FALSE COMMENT 'flag if customer agreed to send review invitation, if needed'";
		$sql[]="ALTER TABLE "._DB_PREFIX_."cart ADD opinus BOOLEAN NOT NULL DEFAULT FALSE COMMENT 'flag if customer agreed to send review invitation, if needed'";
		$this->executeSQL($sql);
		
		if(!parent::install()
			|| !$this->registerHook('extraCarrier')
			|| !$this->registerHook('actionCarrierProcess')
			|| !$this->registerHook('actionValidateOrder')
			|| !$this->registerHook('actionOrderStatusUpdate')
			|| !Configuration::updateValue('OPINUS_DELAYDAYS', 1)
			|| !Configuration::updateValue('OPINUS_ORDER_STATUS_SEND', 5)
			|| !Configuration::updateValue('OPINUS_CONSENT', 0)
			|| !Configuration::updateValue('OPINUS_CONSENT_CHCKD', 1)
		){
			return false;
		} else {
			return true;
		}
	}

	public function uninstall(){
		
		//removing column from database table:
		$sql=array();
		$sql[]="ALTER TABLE "._DB_PREFIX_."orders DROP opinus";
		$sql[]="ALTER TABLE "._DB_PREFIX_."cart DROP opinus";
		$this->executeSQL($sql);
		
		Configuration::deleteByName('OPINUS_KEY');
		Configuration::deleteByName('OPINUS_DELAYDAYS');
		Configuration::deleteByName('OPINUS_ORDER_STATUS_SEND');
		Configuration::deleteByName('OPINUS_ENV');
		Configuration::deleteByName('OPINUS_CONSENT');
		Configuration::deleteByName('OPINUS_CONSENT_CHCKD');
		if(!parent::uninstall()){
			return false;
		} else {
			return true;
		}
	}
	
	public function hookActionOrderStatusUpdate($params){
		$orderstatus2send_invitation=Configuration::get('OPINUS_ORDER_STATUS_SEND');
		$apikey=Configuration::get('OPINUS_KEY');
		$mode=Configuration::get('OPINUS_ENV');
		if(isset($orderstatus2send_invitation) && is_numeric($orderstatus2send_invitation) && $orderstatus2send_invitation>0 && $params['newOrderStatus']->id==$orderstatus2send_invitation && isset($apikey) && strlen($apikey)==20){
			if(isset($mode) && $mode==1){
				$api_url='https://opinus.pl/api/';
			} else {
				$api_url='https://opinus.pl/api-test/';
			}
			
			$order_id=$params['id_order']; //dowolny unikatowy dla sklepu ciąg znaków, do 20 znaków, wymagane; tu liczbowa wartość
			
			//weryfikacja czy jest wyrażona zgoda na wysyłkę danych do zewnętrznego partnera celem prośby o opinię:
			//print_r($params);
			if(isset($params['cart']->opinus)){
				$opinus_consent=$params['cart']->opinus;
			} else {
				$opinus_consent=0;
			}
			
			if($opinus_consent==1){
				$order=new Order($order_id);
				$customer_id=$order->id_customer;
				//$customer_id=$params['id_customer'];
				$customer=new Customer($customer_id); //dane klienta pobiera, tablicę!
				$customer_email=$customer->email; //adres e-mail klienta, wymagany, do wysłania zaproszenia do wystawienia opinii
				$customer_name=$customer->firstname; //imię klienta, opcjonalnie
				$customer_gender_id=$customer->id_gender; // płeć klienta, opcjonalnie, możliwe wartości: m, k
				if($customer_gender_id==1){
					$customer_gender='m';
				}
				if($customer_gender_id==2){
					$customer_gender='k';
				}
				$order_placedate=$order->date_add; //data złożenia zamówienia w formacie: YYYY-MM-DD HH:ii:ss, wymagana
				
				//przygotowanie zestawu danych do wysyłki i wywołanie żądania:
				$dataset = array(
					"authkey" => $apikey,
					"c_email" => $customer_email,
					"c_name" => $customer_name,
					"c_sex" => $customer_gender,
					"o_number" => $order_id,
					"o_placedate" => $order_placedate,
				);
				
				ob_start();
				$ch=curl_init($api_url);
				$data_json=json_encode($dataset);
				curl_setopt($ch, CURLOPT_POSTFIELDS, $data_json);
				curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
				curl_setopt($ch, CURLOPT_POST, true);
				curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
				curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
				
				$request_result=curl_exec($ch); //tu w odpowiedzi w formacie json status (wartość: 0 / 1) i treść odpowiedzi
				curl_close($ch);
				ob_end_flush();
				
				
				$answer=json_decode($request_result, true);
				if(isset($answer['status'])){
					if($answer['status']===1){
						//wysłane poprawnie
						$response='wysłane poprawnie';
					} else {
						//błąd wysyłki
						$response='błąd wysyłki: '.$answer['msg'];
					}
				}
			} else {
			}
			
			//just debug:
			/*
			$myfile=fopen("test_01.txt", "w");
			if(!isset($customer_email)){ $customer_email='brak'; }
			if(!isset($customer_name)){ $customer_name='brak'; }
			if(!isset($customer_gender)){ $customer_gender='brak'; }
			if(!isset($order_placedate)){ $order_placedate='brak'; }
			if(!isset($response)){ $response='brak'; }
			if(!isset($order_id)){ $order_id='brak'; }
			fwrite($myfile, 'zgoda: ['.$opinus_consent.']'."\n".'Odpowiedz api: '.$response."\n".'order_id: '.$order_id.' '."\n".'email: '.$customer_email."\n".'imie: '.$customer_name."\n".'plec: '.$customer_gender."\n".'data: '.$order_placedate."\n");
			fclose($myfile);
			*/
		}
	}
	
	public function hookExtraCarrier($params){
		if(Configuration::get('OPINUS_CONSENT')==1){
			$opinus_consent_chckd=Configuration::get('OPINUS_CONSENT_CHCKD');
			$this->smarty->assign(array('opinus_consent_chckd'=>$opinus_consent_chckd));
			return $this->display(__FILE__, 'opinus-consent.tpl');
		} else {
			$consent_not_needed='<input type="hidden" name="opinus" value="1" />';
			return $consent_not_needed;
		}
	}
	
	public function hookActionCarrierProcess($params){
		$opinus_consent=strval(Tools::getValue('opinus'));
		$opinus_consent_bool=filter_var($opinus_consent, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE);
		
		$cart=$params['cart'];
		$cart->opinus=$opinus_consent_bool;
		$cart->save();
	}
	
	public function hookActionValidateOrder($params){
		
		$opinus_from_cart=filter_var($params['cart']->opinus, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE);
		
		$order=$params['order'];
		
		//cholera wie dlaczego ponizsze nie dziala i nie zapisuje wartosci
		$order->opinus=$opinus_from_cart;
		$order->save();
		
		//obejscie problemu z zapisaniem zgody na wyslanie zaproszenia do ankiety:
		$orderId=$order->id;
		$query='UPDATE '._DB_PREFIX_.'orders SET opinus='.$opinus_from_cart.' WHERE id_order='.$orderId;
		$result=Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS($query);
		
		//just debug:
		/*
		$myfile=fopen("test_02.txt", "w");
		$serializedData=serialize($params);
		fwrite($myfile, 'parametry Order po zapisie:'."\n".$serializedData."\n\n".'parametr Opinus w Cart, pobrany z parametrow: '.$params['cart']->opinus."\n\n".'parametr Opinus w Order po zapisie: '.$order->opinus.''."\n\n".'id zamowienia: '.$orderId);
		fclose($myfile);
		*/
	}
	
	public function getContent(){
		$output1=null;
		$output2=null;
		if(Tools::isSubmit('submit'.$this->name.'api')){
			$op_api_key=strval(Tools::getValue('OPINUS_KEY'));
			if(isset($op_api_key) && !empty($op_api_key)){
				if(!Validate::isGenericName($op_api_key) || strlen($op_api_key)!=20){
					$output1.=$this->displayError($this->l('Invalid Opinus api key value'));
				} else {
					if($op_api_key!=Configuration::get('OPINUS_KEY')){
						Configuration::updateValue('OPINUS_KEY', $op_api_key);
						$output1.=$this->displayConfirmation($this->l('API key has been updated.'));
					}
				}
			}
			$op_environment=strval(Tools::getValue('OPINUS_ENV'));
			if(isset($op_environment)){
				if($op_environment!=0 && $op_environment!='' && $op_environment!=1){
					$output1.=$this->displayError($this->l('Incorrect environment type value.'));
				} else {
					Configuration::updateValue('OPINUS_ENV', $op_environment);
					$output1.=$this->displayConfirmation($this->l('Opinus mode has been updated.'));
				}
			}
		}
		if(Tools::isSubmit('submit'.$this->name.'settings')){
			$op_delay=strval(Tools::getValue('OPINUS_DELAYDAYS'));
			if(isset($op_delay)){
				if(!is_numeric($op_delay) || $op_delay<0){
					$output2.=$this->displayError($this->l('Incorrect number of delay (days).'));
				} else {
					if($op_delay!=Configuration::get('OPINUS_DELAYDAYS')){
						Configuration::updateValue('OPINUS_DELAYDAYS', $op_delay);
						$output2.=$this->displayConfirmation($this->l('Invitation delay time has been updated.'));
					}
				}
			}
			$op_orderstatus=strval(Tools::getValue('OPINUS_ORDER_STATUS_SEND'));
			if(isset($op_orderstatus)){
				if(!$op_orderstatus || empty($op_orderstatus) || !is_numeric($op_orderstatus) || $op_orderstatus<0){
					$output2.=$this->displayError($this->l('Invalid chosen order status.'));
				} else {
					if($op_orderstatus!=Configuration::get('OPINUS_ORDER_STATUS_SEND')){
						Configuration::updateValue('OPINUS_ORDER_STATUS_SEND', $op_orderstatus);
						$output2.=$this->displayConfirmation($this->l('Order status to send invitation for writing review has been updated.'));
					}
				}
			}
			$op_extraconsent=strval(Tools::getValue('OPINUS_CONSENT'));
			if(isset($op_extraconsent)){
				if($op_extraconsent!=0 && $op_extraconsent!='' && $op_extraconsent!=1){
					$output2.=$this->displayError($this->l('Invalid chosen option.'));
				} else {
					if($op_extraconsent!=Configuration::get('OPINUS_CONSENT')){
						Configuration::updateValue('OPINUS_CONSENT', $op_extraconsent);
						if($op_extraconsent==0){
							$output2.=$this->displayConfirmation($this->l('From now customers will NOT see extra checkbox to agree receiving invitation to write review.'));
						}
						if($op_extraconsent==1){
							$output2.=$this->displayConfirmation($this->l('From now customers will see extra checkbox to agree receiving invitation to write review.'));
						}
					}
				}
			}
			$op_consentchckd=strval(Tools::getValue('OPINUS_CONSENT_CHCKD'));
			if(isset($op_consentchckd)){
				if($op_consentchckd!=0 && $op_consentchckd!='' && $op_consentchckd!=1){
					$output2.=$this->displayError($this->l('Invalid chosen option.'));
				} else {
					if($op_consentchckd!=Configuration::get('OPINUS_CONSENT_CHCKD')){
						Configuration::updateValue('OPINUS_CONSENT_CHCKD', $op_consentchckd);
						if($op_consentchckd==0){
							$output2.=$this->displayConfirmation($this->l('Consent will NOT be checked by default.'));
						}
						if($op_consentchckd==1){
							$output2.=$this->displayConfirmation($this->l('Consent will be checked by default.'));
						}
					}
				}
			}
		}
		
		$this->_html .= $this->_showInfo();
		$this->_html .= $output1.$this->opinusApiKeyForm();
		$this->_html .= $output2.$this->opinusSettings();
		return $this->_html;
	}
	
	protected function _showInfo(){
		return $this->display(__FILE__, 'opinus-info.tpl');
	}
	
	public function opinusApiKeyForm(){
		//Get default language
		$default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
		
		//Init fields form array
		$fields_form[0]['form']=array(
			'legend' => array(
				'title' => $this->l('Communication with Opinus.pl'),
				'icon' => 'icon-cogs'
			),
			'input' => array(
				array(
					'type' => 'text',
					'label' => $this->l('Opinus.pl API key'),
					'desc' => $this->l('Opinus API key is available after siging in opinus.pl to managae your store. 20 chars.'),
					'name' => 'OPINUS_KEY',
					'required' => true,
					'maxlength' => 20,
					'placeholder' => 'abc123def456ghi789jk',
					'prefix' => '<i class="icon icon-key"></i>'
				),
				array(
					'type' => 'switch',
					'label' => $this->l('Production mode'),
					'name' => 'OPINUS_ENV',
					'required'  => true,
					'is_bool' => true,
					'desc' => $this->l('In test mode data is send to Opinus api but finally not save in portal.'),
					'values' => array(
						array(
							'id' => 'prd',
							'value' => 1,
							'label' => $this->l('production')
						),
						array(
							'id' => 'dev',
							'value' => 0,
							'label' => $this->l('test')
						)
					),
				),
			),
			'submit' => array(
				'title' => $this->l('Save')
			)
		);
		
		$helper1=new HelperForm();
		
		//Module, token and currentIndex
		$helper1->module = $this;
		$helper1->name_controller = $this->name;
		$helper1->token = Tools::getAdminTokenLite('AdminModules');
		$helper1->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
		
		//Language
		$helper1->default_form_language = $default_lang;
		$helper1->allow_employee_form_lang = $default_lang;
		
		//Title and toolbar
		$helper1->title = $this->displayName;
		$helper1->show_toolbar = true;        // false -> remove toolbar
		$helper1->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper1->submit_action = 'submit'.$this->name.'api';
		$helper1->toolbar_btn = array(
			'save' =>
			array(
				'desc' => $this->l('Save'),
				'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
				'&token='.Tools::getAdminTokenLite('AdminModules'),
			),
			'back' => array(
				'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
				'desc' => $this->l('Back to list')
			)
		);
		
		//Load current value
		$helper1->fields_value['OPINUS_KEY'] = Configuration::get('OPINUS_KEY');
		$helper1->fields_value['OPINUS_ENV'] = Configuration::get('OPINUS_ENV');
		
		return $helper1->generateForm($fields_form);
	}
	
	public function opinusSettings(){
		//Get default language
		$default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
		
		$order_statuses=OrderState::getOrderStates((int)$this->context->language->id);
		$statuses[]=array('order_state_id'=>'0', 'order_state_name'=>'-- '.$this->l('choose status').' --');
		foreach($order_statuses as $status){
			//$statuses[$status['id_order_state']]=$status['name'];
			if($status['hidden']==0 && $status['deleted']==0){
				$statuses[]=array('order_state_id'=>$status['id_order_state'], 'order_state_name'=>$status['name'].' (id: '.$status['id_order_state'].')');
			}
		}
		
		//Init fields form array
		$fields_form[0]['form']=array(
			'legend' => array(
				'title' => $this->l('Review invitations settings'),
				'icon' => 'icon-cogs'
			),
			'input' => array(
				array(
					'type' => 'select',
					'label' => $this->l('The status after which we want to send the invitation'),
					'desc' => $this->l('When order status will change to this status - then data will be send to Opinus portal to send invitation on next step.'),
					'name' => 'OPINUS_ORDER_STATUS_SEND',
					'options' => array(
						'query' =>  $statuses,
						'id' => 'order_state_id',
						'name' => 'order_state_name',
					),
					'required' => true,
					'prefix' => '<i class="icon icon-key"></i>'
				),
				array(
					'type' => 'text',
					'label' => $this->l('Delays (in days)'),
					'desc' => $this->l('Number of days counted from the moment when order status has changed to defined (for example "delivered"). After this time the invitation for writing review will be sent. 0 = no delay.'),
					'name' => 'OPINUS_DELAYDAYS',
					'required' => false,
					'maxlength' => 2,
					'prefix' => '<i class="icon icon-key"></i>'
				),
				array(
					'type' => 'switch',
					'label' => $this->l('Show extra consent in cart'),
					'name' => 'OPINUS_CONSENT',
					'required'  => false,
					'is_bool' => true,
					'desc' => $this->l('Switch on if you would like to present in cart extra consent for passing customer data to partner - Opinus.'),
					'values' => array(
						array(
							'id' => 'show',
							'value' => 1,
							'label' => $this->l('show')
						),
						array(
							'id' => 'notshow',
							'value' => 0,
							'label' => $this->l('do not show')
						)
					),
				),
				array(
					'type' => 'switch',
					'label' => $this->l('Set consent checked by default'),
					'name' => 'OPINUS_CONSENT_CHCKD',
					'required'  => false,
					'is_bool' => true,
					'desc' => $this->l('Switch on if you would like to have checked above consent at start, of course if it is presented.'),
					'values' => array(
						array(
							'id' => 'chked',
							'value' => 1,
							'label' => $this->l('checked')
						),
						array(
							'id' => 'unchked',
							'value' => 0,
							'label' => $this->l('not checked')
						)
					),
				)
			),
			'submit' => array(
				'title' => $this->l('Save')
			)
		);
		$helper2=new HelperForm();
		
		if(!Configuration::get('OPINUS_ORDER_STATUS_SEND')){
			$helper2->fields_value['OPINUS_ORDER_STATUS_SEND']=5; //Ustawia wartość pola o danej nazwie. Tutaj domyślną wartość pola select
		} else {
			$helper2->fields_value['OPINUS_ORDER_STATUS_SEND']=Configuration::get('OPINUS_ORDER_STATUS_SEND');
		}
		if(!Configuration::get('OPINUS_DELAYDAYS')){
			$helper2->fields_value['OPINUS_DELAYDAYS']=0; //Ustawia wartość pola o danej nazwie gdy brak jeszcze skonfigurowanej. Tutaj domyślną wartość pola select
		} else {
			$helper2->fields_value['OPINUS_DELAYDAYS']=Configuration::get('OPINUS_DELAYDAYS');
		}
		$helper2->fields_value['OPINUS_CONSENT'] = Configuration::get('OPINUS_CONSENT');
		$helper2->fields_value['OPINUS_CONSENT_CHCKD'] = Configuration::get('OPINUS_CONSENT_CHCKD');
		
		//Module, token and currentIndex
		$helper2->module = $this;
		$helper2->name_controller=$this->name;
		$helper2->token = Tools::getAdminTokenLite('AdminModules');
		$helper2->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
		
		//Language
		$helper2->default_form_language = $default_lang;
		$helper2->allow_employee_form_lang = $default_lang;
		
		//Title and toolbar
		$helper2->title = $this->displayName;
		$helper2->show_toolbar = true;        // false -> remove toolbar
		$helper2->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper2->submit_action = 'submit'.$this->name.'settings';
		$helper2->toolbar_btn = array(
			'save' =>
			array(
				'desc' => $this->l('Save'),
				'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
				'&token='.Tools::getAdminTokenLite('AdminModules'),
			),
			'back' => array(
				'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
				'desc' => $this->l('Back to list')
			)
		);
		
		return $helper2->generateForm($fields_form);
	}

}

?>